@@ -14,6 +14,7 @@
 #                       ca-ecc384-cert.der
 #                       server-cert.pem
 #                       server-cert.der
+#                       server-cert-chain.der
 #                       server-ecc-rsa.pem
 #                       server-ecc.pem
 #                       1024/client-cert.der
@@ -127,25 +128,7 @@ run_renewcerts(){
     mv tmp.pem client-cert.pem
     echo "End of section"
     echo "---------------------------------------------------------------------"
-    ############################################################
-    #### update the self-signed (3072-bit) client-cert.pem #####
-    ############################################################
-    echo "Updating 3072-bit client-cert.pem"
-    echo ""
-    #pipe the following arguments to openssl req...
-    echo -e "US\\nMontana\\nBozeman\\nwolfSSL_3072\\nProgramming-3072\\nwww.wolfssl.com\\ninfo@wolfssl.com\\n.\\n.\\n" | openssl req -new -newkey rsa:3072 -keyout client-key-3072.pem -config ./wolfssl.cnf -nodes -out client-cert-3072.csr
-    check_result $? "Step 1"
 
-
-    openssl x509 -req -in client-cert-3072.csr -days 1000 -extfile wolfssl.cnf -extensions wolfssl_opts -signkey client-key-3072.pem -out client-cert-3072.pem
-    check_result $? "Step 2"
-    rm client-cert-3072.csr
-
-    openssl x509 -in client-cert-3072.pem -text > tmp.pem
-    check_result $? "Step 3"
-    mv tmp.pem client-cert-3072.pem
-    echo "End of section"
-    echo "---------------------------------------------------------------------"
     ############################################################
     #### update the self-signed (1024-bit) client-cert.pem #####
     ############################################################
@@ -165,6 +148,30 @@ run_renewcerts(){
     mv ./1024/tmp.pem ./1024/client-cert.pem
     echo "End of section"
     echo "---------------------------------------------------------------------"
+    ############################################################
+    #### update the self-signed (3072-bit) client-cert.pem #####
+    ############################################################
+    echo "Updating 3072-bit client-cert.pem"
+    echo ""
+    #pipe the following arguments to openssl req...
+    echo -e "US\\nMontana\\nBozeman\\nwolfSSL_3072\\nProgramming-3072\\nwww.wolfssl.com\\ninfo@wolfssl.com\\n.\\n.\\n" | openssl req -new -key ./3072/client-key.pem -config ./wolfssl.cnf -nodes -out ./3072/client-cert.csr
+    check_result $? "Step 1"
+
+    openssl x509 -req -in ./3072/client-cert.csr -days 1000 -extfile wolfssl.cnf -extensions wolfssl_opts -signkey ./3072/client-key.pem -out ./3072/client-cert.pem
+    check_result $? "Step 2"
+    rm ./3072/client-cert.csr
+
+    openssl x509 -in ./3072/client-cert.pem -text > ./3072/tmp.pem
+    check_result $? "Step 3"
+    mv ./3072/tmp.pem ./3072/client-cert.pem
+
+    openssl rsa -in ./3072/client-key.pem -outform der -out ./3072/client-key.der
+    openssl rsa -inform pem -in ./3072/client-key.pem -outform der -out ./3072/client-keyPub.der -pubout
+    openssl x509 -in ./3072/client-cert.pem -outform der -out ./3072/client-cert.der
+
+    echo "End of section"
+    echo "---------------------------------------------------------------------"
+
     ############################################################
     #### update the self-signed (4096-bit) client-cert.pem #####
     ############################################################
@@ -174,7 +181,6 @@ run_renewcerts(){
     echo -e "US\\nMontana\\nBozeman\\nwolfSSL_4096\\nProgramming-4096\\nwww.wolfssl.com\\ninfo@wolfssl.com\\n.\\n.\\n" | openssl req -new -key ./4096/client-key.pem -config ./wolfssl.cnf -nodes -out ./4096/client-cert.csr
     check_result $? "Step 1"
 
-
     openssl x509 -req -in ./4096/client-cert.csr -days 1000 -extfile wolfssl.cnf -extensions wolfssl_opts -signkey ./4096/client-key.pem -out ./4096/client-cert.pem
     check_result $? "Step 2"
     rm ./4096/client-cert.csr
@@ -182,8 +188,13 @@ run_renewcerts(){
     openssl x509 -in ./4096/client-cert.pem -text > ./4096/tmp.pem
     check_result $? "Step 3"
     mv ./4096/tmp.pem ./4096/client-cert.pem
+
+    openssl rsa -in ./4096/client-key.pem -outform der -out ./4096/client-key.der
+    openssl rsa -inform pem -in ./4096/client-key.pem -outform der -out ./4096/client-keyPub.der -pubout
+    openssl x509 -in ./4096/client-cert.pem -outform der -out ./4096/client-cert.der
     echo "End of section"
     echo "---------------------------------------------------------------------"
+
     ############################################################
     ########## update the self-signed ca-cert.pem ##############
     ############################################################
@@ -487,8 +498,21 @@ run_renewcerts(){
     check_result $? "Der Cert 11"
     openssl x509 -inform PEM -in server-ecc-comp.pem -outform DER -out server-ecc-comp.der
     check_result $? "Der Cert 12"
+    cat server-cert.der ca-cert.der >server-cert-chain.der
+    check_result $? "Der Cert 13"
     echo "End of section"
     echo "---------------------------------------------------------------------"
+
+    ############################################################
+    ########## generate PKCS7 bundles ##########################
+    ############################################################
+    echo "Renewing Ed448 certificates"
+    cd ed448
+    ./gen-ed448-certs.sh
+    cd ..
+    echo "End of section"
+    echo "---------------------------------------------------------------------"
+
     ############################################################
     ###### update the ecc-rsa-server.p12 file ##################
     ############################################################
